/*
 * Copyright (c) 2018-2020, NVIDIA CORPORATION. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

/*
 * t19x-nvlink-endpt.h:
 * This header contains the structures and APIs needed by the Tegra NVLINK
 * endpoint driver.
 */

#ifndef COMMON_T19X_NVLINK_ENDPT_H
#define COMMON_T19X_NVLINK_ENDPT_H

#include <nvlink/common/tegra-nvlink.h>
#include <nvlink/common/log.h>
#include <nvlink/common/timer.h>
#include <nvlink/common/io.h>
#include <nvlink/common/clk.h>
#include <nvlink/common/reset.h>
#include <nvlink/common/irq.h>
#include <nvlink/common/fw.h>
#include <nvlink/common/prod.h>
#include <nvlink/common/kmem.h>

#define NVLINK_MODULE_NAME			"t19x-nvlink-endpt"
#define NVLINK_IP_VERSION			2 /* NVLINK VERSION 2.0 */
#define DEFAULT_IS_NEA				0

enum nvlink_refclk {
	NVLINK_REFCLK_150,
	NVLINK_REFCLK_156
};

/*
 * This structure is used for storing parameters which describe the Single-Lane
 * (SL / 1/8th) mode policy. A few acronyms that are used in this structure are
 * as follows:
 *    - SL = Single-Lane / 1/8th mode - sublink low power mode where only 1 of
 *           the 8 lanes is used
 *    - FB = Full Bandwidth (i.e. HISPEED mode)
 *    - LP = Low Power (i.e. SL / 1/8th mode)
 *    - IC = Idle Counter - the idle counter is used to monitor traffic per
 *           sub-link
 */
struct single_lane_params {
	/* Idle counter increment in FB */
	u16 fb_ic_inc;

	/* Idle counter increment in LP */
	u16 lp_ic_inc;

	/* Idle counter decrement in FB */
	u16 fb_ic_dec;

	/* Idle counter decrement in LP */
	u16 lp_ic_dec;

	/* SL entry threshold */
	u32 enter_thresh;

	/* SL exit threshold */
	u32 exit_thresh;

	/* Idle counter saturation limit */
	u32 ic_limit;
};

/* Tegra endpoint driver's private link struct */
struct tnvlink_link {
	/* TLC errors status. TODO: Add more description here */
	u32 tlc_tx_err_status0;
	u32 tlc_rx_err_status0;
	u32 tlc_rx_err_status1;
	/* Successful error recoveries */
	u32 error_recoveries;
	/* Parameters which describe the selected Single-Lane policy */
	struct single_lane_params sl_params;
	/* Pointer to parent struct tnvlink_dev */
	struct tnvlink_dev *tdev;
	/* Pointer to parent struct nvlink_link */
	struct nvlink_link *nlink;
};

/* Tegra endpoint driver's private device struct */
struct tnvlink_dev {
	/* Are we using the RM shim driver? */
	bool rm_shim_enabled;
	/* irq below represents the interrupt line going to GIC and LIC */
	int irq;
	/* clocks */
	struct nvlink_clk clk_nvhs_pll0_mgmt;
	struct nvlink_clk clk_pllrefe_vcoout_gated;
	struct nvlink_clk clk_nvlink_sys;
	struct nvlink_clk clk_pllnvhs;
	struct nvlink_clk clk_m;
	struct nvlink_clk clk_nvlink_pll_txclk;
	struct nvlink_clk clk_nvlink_tx;
	/* resets */
	struct nvlink_reset_control rst_mssnvl;
	struct nvlink_reset_control rst_nvhs_uphy_pm;
	struct nvlink_reset_control rst_nvhs_uphy;
	struct nvlink_reset_control rst_nvhs_uphy_pll0;
	struct nvlink_reset_control rst_nvhs_uphy_l0;
	struct nvlink_reset_control rst_nvhs_uphy_l1;
	struct nvlink_reset_control rst_nvhs_uphy_l2;
	struct nvlink_reset_control rst_nvhs_uphy_l3;
	struct nvlink_reset_control rst_nvhs_uphy_l4;
	struct nvlink_reset_control rst_nvhs_uphy_l5;
	struct nvlink_reset_control rst_nvhs_uphy_l6;
	struct nvlink_reset_control rst_nvhs_uphy_l7;
	struct nvlink_reset_control rst_nvlink;
	/* Powergate id */
	int pgid_nvl;
	struct nvlink_tegra_prod prod_list;
	bool is_nea;
	/* Nvlink refclk*/
	enum nvlink_refclk refclk;
	bool is_tp_cntr_running;
	struct tnvlink_link tlink;
	struct nvlink_device *ndev;
};

extern const struct single_lane_params entry_100us_sl_params;
extern const struct file_operations t19x_nvlink_endpt_ops;

int t19x_nvlink_dev_early_init(struct nvlink_device *ndev);
int t19x_nvlink_dev_interface_init(struct nvlink_device *ndev);
int t19x_nvlink_dev_reg_init(struct nvlink_device *ndev);
int t19x_nvlink_link_early_init(struct nvlink_device *ndev);
int t19x_nvlink_dev_car_disable(struct nvlink_device *ndev);
int t19x_nvlink_suspend(struct device *dev);

int t19x_nvlink_check_fuse(void);
void t19x_nvlink_ndev_tdev_init(struct nvlink_device *ndev,
					struct tnvlink_dev *tdev);
int t19x_nvlink_register_endpt(struct nvlink_device *ndev);

int wait_for_reg_cond_nvlink(
			struct tnvlink_dev *tdev,
			u32 reg,
			u32 bit_mask,
			bool check_for_bit_set,
			char *bit_name,
			u32 (*reg_readl)(struct tnvlink_dev *, u32),
			u32 *reg_val,
			u32 timeout_us);

int t19x_nvlink_dev_interface_disable(struct nvlink_device *ndev);

void minion_dump_pc_trace(struct tnvlink_dev *tdev);
void minion_dump_registers(struct tnvlink_dev *tdev);
int minion_boot(struct tnvlink_dev *tdev);
int init_nvhs_phy(struct tnvlink_dev *tdev);
int minion_send_cmd(struct tnvlink_dev *tdev,
				u32 cmd,
				u32 scratch0_val);
void nvlink_enable_AN0_packets(struct tnvlink_dev *tdev);

void nvlink_config_minion_falcon_intr(struct tnvlink_dev *tdev);
void nvlink_config_common_intr(struct tnvlink_dev *tdev);
void nvlink_enable_dl_interrupts(struct tnvlink_dev *tdev);
void nvlink_enable_link_interrupts(struct tnvlink_dev *tdev);
void nvlink_disable_link_interrupts(struct tnvlink_dev *tdev);
void minion_service_falcon_intr(struct tnvlink_dev *tdev);
void nvlink_disable_dl_interrupts(struct tnvlink_dev *tdev);

void init_single_lane_params(struct tnvlink_dev *tdev);
u32 t19x_nvlink_get_link_state(struct nvlink_device *ndev);
u32 t19x_nvlink_get_link_mode(struct nvlink_device *ndev);
int t19x_nvlink_set_link_mode(struct nvlink_device *ndev, u32 mode);
void t19x_nvlink_get_tx_sublink_state(struct nvlink_device *ndev,
				u32 *tx_sublink_state);
void t19x_nvlink_get_rx_sublink_state(struct nvlink_device *ndev,
				u32 *rx_sublink_state);
u32 t19x_nvlink_get_sublink_mode(struct nvlink_device *ndev,
				bool is_rx_sublink);
int t19x_nvlink_set_sublink_mode(struct nvlink_device *ndev, bool is_rx_sublink,
				u32 mode);
bool is_link_connected(struct tnvlink_link *tlink);
int nvlink_retrain_link(struct tnvlink_dev *tdev, bool from_off);
int t19x_nvlink_write_discovery_token(struct tnvlink_dev *tdev, u64 token);
int t19x_nvlink_read_discovery_token(struct tnvlink_dev *tdev, u64 *token);
int t19x_nvlink_reset_tp_counters(struct tnvlink_dev *tdev);
int t19x_nvlink_freeze_tp_counters(struct tnvlink_dev *tdev, bool bFreeze);
int t19x_nvlink_config_tp_counters(struct tnvlink_dev *tdev);
int t19x_nvlink_get_tp_counters(struct tnvlink_dev *tdev, u64 *tx0cnt,
					u64 *tx1cnt, u64 *rx0cnt, u64 *rx1cnt);
#endif /* COMMON_T19X_NVLINK_ENDPT_H */
